pub enum Constraint<T: Types> {
Pred(Pred<T>, Option<T::Tag>),
Conj(Vec<Self>),
ForAll(Bind<T>, Box<Self>),
}Variants
Implementations
Sourceimpl<T: Types> Constraint<T>
impl<T: Types> Constraint<T>
pub const TRUE: Self
pub fn foralls(bindings: Vec<Bind<T>>, c: Self) -> Self
pub fn conj(cstrs: Vec<Self>) -> Self
Sourcepub fn concrete_head_count(&self) -> usize
pub fn concrete_head_count(&self) -> usize
Returns true if the constraint has at least one concrete RHS (“head”) predicates.
If !c.is_concrete then c is trivially satisfiable and we can avoid calling fixpoint.
Returns the number of concrete, non-trivial head predicates in the constraint.
Sourcepub fn flatten<F1>(&self, is_underscore: F1) -> Vec<FlatConstraint<T>>
pub fn flatten<F1>(&self, is_underscore: F1) -> Vec<FlatConstraint<T>>
Flattens a single constraint into a list of individual constraints which may be checked for satisfiability.
NOTE: Assumes and requires that all binder names are unique, i.e. there is no shadowing (it is OK to have multiple binders of the same name so long as they never are used, e.g. UNDERSCORE).